@@ -1,27 +1,24 @@
-use crate::context::{consts, DataType};
 use crate::core::*;
 
 /// The basic data type used for each index in an element buffer.
-pub trait ElementBufferDataType:
-    Default + std::fmt::Debug + Clone + internal::BufferDataTypeExtension
-{
+pub trait ElementBufferDataType: data_type::DataType {
     ///
     /// Converts the index to `u32`.
     ///
-    fn into_u32(&self) -> u32;
+    fn as_u32(&self) -> u32;
 }
 impl ElementBufferDataType for u8 {
-    fn into_u32(&self) -> u32 {
+    fn as_u32(&self) -> u32 {
         *self as u32
     }
 }
 impl ElementBufferDataType for u16 {
-    fn into_u32(&self) -> u32 {
+    fn as_u32(&self) -> u32 {
         *self as u32
     }
 }
 impl ElementBufferDataType for u32 {
-    fn into_u32(&self) -> u32 {
+    fn as_u32(&self) -> u32 {
         *self
     }
 }
@@ -35,79 +32,106 @@ pub struct ElementBuffer {
     context: Context,
     id: crate::context::Buffer,
     count: usize,
-    data_type: DataType,
+    data_type: u32,
 }
 
 impl ElementBuffer {
     ///
     /// Creates a new empty element buffer.
     ///
-    pub fn new<T: ElementBufferDataType>(context: &Context) -> ThreeDResult<ElementBuffer> {
-        let id = context.create_buffer().unwrap();
-        Ok(ElementBuffer {
+    pub fn new(context: &Context) -> ThreeDResult<Self> {
+        let id = unsafe {
+            context
+                .create_buffer()
+                .map_err(|e| CoreError::BufferCreation(e))?
+        };
+        Ok(Self {
             context: context.clone(),
             id,
             count: 0,
-            data_type: T::data_type(),
+            data_type: 0,
         })
     }
 
     ///
     /// Creates a new element buffer and fills it with the given indices which must be divisable by 3.
     ///
-    pub fn new_with<T: ElementBufferDataType>(
+    pub fn new_with_data<T: ElementBufferDataType>(
         context: &Context,
         data: &[T],
-    ) -> ThreeDResult<ElementBuffer> {
-        let mut buffer = Self::new::<T>(context)?;
+    ) -> ThreeDResult<Self> {
+        let mut buffer = Self::new(context)?;
         if data.len() > 0 {
-            buffer.fill_with(data)?;
+            buffer.fill(data)?;
         }
         Ok(buffer)
     }
-    ///
-    /// Fills the buffer with the given indices which must be divisable by 3.
-    ///
-    pub fn fill_with<T: ElementBufferDataType>(&mut self, data: &[T]) -> ThreeDResult<()> {
-        if data.len() % 3 != 0 {
-            Err(CoreError::InvalidBufferLength(
-                "index".to_string(),
-                data.len(),
-            ))?;
-        }
 
-        self.bind();
-        T::buffer_data(
-            &self.context,
-            consts::ELEMENT_ARRAY_BUFFER,
-            data,
-            consts::STATIC_DRAW,
-        );
-        self.data_type = T::data_type();
-        self.context.unbind_buffer(consts::ELEMENT_ARRAY_BUFFER);
-        self.count = data.len();
-        Ok(())
+    ///
+    /// Creates a new element buffer and fills it with the given indices which must be divisable by 3.
+    ///
+    #[deprecated = "use new_with_data()"]
+    pub fn new_with<T: ElementBufferDataType>(context: &Context, data: &[T]) -> ThreeDResult<Self> {
+        Self::new_with_data(context, data)
     }
 
     ///
-    /// The number of elements in the buffer.
+    /// Fills the buffer with the given indices which must be divisable by 3.
+    ///
+    pub fn fill<T: ElementBufferDataType>(&mut self, data: &[T]) -> ThreeDResult<()> {
+        self.bind();
+        unsafe {
+            self.context.buffer_data_u8_slice(
+                crate::context::ELEMENT_ARRAY_BUFFER,
+                to_byte_slice(data),
+                crate::context::STATIC_DRAW,
+            );
+            self.context
+                .bind_buffer(crate::context::ELEMENT_ARRAY_BUFFER, None);
+        }
+        self.count = data.len();
+        self.data_type = T::data_type();
+        self.context.error_check()
+    }
+
+    ///
+    /// Fills the buffer with the given indices which must be divisable by 3.
+    ///
+    #[deprecated = "use fill()"]
+    pub fn fill_with<T: ElementBufferDataType>(&mut self, data: &[T]) -> ThreeDResult<()> {
+        self.fill(data)
+    }
+
+    ///
+    /// The number of values in the buffer.
     ///
     pub fn count(&self) -> usize {
         self.count
     }
 
-    pub(crate) fn data_type(&self) -> DataType {
-        self.data_type
+    ///
+    /// The number of triangles in the buffer.
+    ///
+    pub fn triangle_count(&self) -> usize {
+        self.count / 3
     }
 
     pub(crate) fn bind(&self) {
-        self.context
-            .bind_buffer(consts::ELEMENT_ARRAY_BUFFER, &self.id);
+        unsafe {
+            self.context
+                .bind_buffer(crate::context::ELEMENT_ARRAY_BUFFER, Some(self.id));
+        }
+    }
+
+    pub(crate) fn data_type(&self) -> u32 {
+        self.data_type
     }
 }
 
 impl Drop for ElementBuffer {
     fn drop(&mut self) {
-        self.context.delete_buffer(&self.id);
+        unsafe {
+            self.context.delete_buffer(self.id);
+        }
     }
 }
