@@ -18,7 +18,6 @@ mod render_target_cube_map;
 #[doc(inline)]
 pub use render_target_cube_map::*;
 
-use crate::context::consts;
 use crate::core::*;
 
 ///
@@ -100,33 +99,14 @@ impl Default for ClearState {
     }
 }
 
-///
-/// The destination of applying a copy.
-///
-pub enum CopyDestination<'a, 'b, 'c, 'd, T: TextureDataType> {
-    /// Copies to the [Screen].
-    Screen,
-    /// Copies to a [Texture2D].
-    ColorTexture(&'d mut Texture2D<T>),
-    /// Copies to a [DepthTargetTexture2D].
-    DepthTexture(&'d mut DepthTargetTexture2D),
-    /// Copies to a [RenderTarget].
-    RenderTarget(&'c RenderTarget<'a, 'b, T>),
-}
-
 pub(in crate::core) fn new_framebuffer(
     context: &Context,
 ) -> ThreeDResult<crate::context::Framebuffer> {
-    Ok(context
-        .create_framebuffer()
-        .ok_or(CoreError::RenderTargetCreation)?)
-}
-
-#[cfg(feature = "debug")]
-fn check(context: &Context) -> ThreeDResult<()> {
-    context
-        .check_framebuffer_status()
-        .or_else(|status| Err(CoreError::RenderTargetCreation))
+    unsafe {
+        Ok(context
+            .create_framebuffer()
+            .map_err(|e| CoreError::RenderTargetCreation(e))?)
+    }
 }
 
 fn clear(context: &Context, clear_state: &ClearState) {
@@ -144,32 +124,34 @@ fn clear(context: &Context, clear_state: &ClearState) {
         || clear_state.green.is_some()
         || clear_state.blue.is_some()
         || clear_state.alpha.is_some();
-    if clear_color {
-        context.clear_color(
-            clear_state.red.unwrap_or(0.0),
-            clear_state.green.unwrap_or(0.0),
-            clear_state.blue.unwrap_or(0.0),
-            clear_state.alpha.unwrap_or(1.0),
-        );
-    }
-    if let Some(depth) = clear_state.depth {
-        context.clear_depth(depth);
-    }
-    context.clear(if clear_color && clear_state.depth.is_some() {
-        consts::COLOR_BUFFER_BIT | consts::DEPTH_BUFFER_BIT
-    } else {
+    unsafe {
         if clear_color {
-            consts::COLOR_BUFFER_BIT
-        } else {
-            consts::DEPTH_BUFFER_BIT
+            context.clear_color(
+                clear_state.red.unwrap_or(0.0),
+                clear_state.green.unwrap_or(0.0),
+                clear_state.blue.unwrap_or(0.0),
+                clear_state.alpha.unwrap_or(1.0),
+            );
         }
-    });
+        if let Some(depth) = clear_state.depth {
+            context.clear_depth_f32(depth);
+        }
+        context.clear(if clear_color && clear_state.depth.is_some() {
+            crate::context::COLOR_BUFFER_BIT | crate::context::DEPTH_BUFFER_BIT
+        } else {
+            if clear_color {
+                crate::context::COLOR_BUFFER_BIT
+            } else {
+                crate::context::DEPTH_BUFFER_BIT
+            }
+        });
+    }
 }
 
 fn copy_from(
     context: &Context,
-    color_texture: Option<&impl Texture>,
-    depth_texture: Option<&impl Texture>,
+    color_texture: Option<&Texture2D>,
+    depth_texture: Option<&DepthTargetTexture2D>,
     viewport: Viewport,
     write_mask: WriteMask,
 ) -> ThreeDResult<()> {
@@ -209,7 +191,7 @@ fn copy_from(
                 effect.use_texture("colorMap", tex)?;
             }
             if let Some(tex) = depth_texture {
-                effect.use_texture("depthMap", tex)?;
+                effect.use_depth_texture("depthMap", tex)?;
             }
             effect.apply(
                 RenderStates {
@@ -233,8 +215,8 @@ fn copy_from(
 
 fn copy_from_array(
     context: &Context,
-    color_texture: Option<(&impl TextureArray, u32)>,
-    depth_texture: Option<(&impl TextureArray, u32)>,
+    color_texture: Option<(&Texture2DArray, u32)>,
+    depth_texture: Option<(&DepthTargetTexture2DArray, u32)>,
     viewport: Viewport,
     write_mask: WriteMask,
 ) -> ThreeDResult<()> {
@@ -276,11 +258,11 @@ fn copy_from_array(
         context.effect(fragment_shader_source, |effect| {
             if let Some((tex, layer)) = color_texture {
                 effect.use_texture_array("colorMap", tex)?;
-                effect.use_uniform_int("colorLayer", &(layer as i32))?;
+                effect.use_uniform("colorLayer", layer as i32)?;
             }
             if let Some((tex, layer)) = depth_texture {
-                effect.use_texture_array("depthMap", tex)?;
-                effect.use_uniform_int("depthLayer", &(layer as i32))?;
+                effect.use_depth_texture_array("depthMap", tex)?;
+                effect.use_uniform("depthLayer", layer as i32)?;
             }
             effect.apply(
                 RenderStates {
