@@ -14,7 +14,7 @@ pub struct GUI {
     height: u32,
     program: Program,
     texture_version: u64,
-    texture: Option<Texture2D<u8>>,
+    texture: Option<Texture2D>,
 }
 
 impl GUI {
@@ -123,16 +123,12 @@ impl GUI {
         if self.texture.is_none() || self.texture_version != egui_texture.version {
             let mut pixels = Vec::new();
             for pixel in egui_texture.srgba_pixels() {
-                pixels.push(pixel.r());
-                pixels.push(pixel.g());
-                pixels.push(pixel.b());
-                pixels.push(pixel.a());
+                pixels.push([pixel.r(), pixel.g(), pixel.b(), pixel.a()]);
             }
             self.texture = Some(Texture2D::new(
                 &self.context,
-                &CPUTexture {
-                    data: pixels,
-                    format: Format::RGBA,
+                &CpuTexture {
+                    data: TextureData::RgbaU8(pixels),
                     width: egui_texture.width as u32,
                     height: egui_texture.height as u32,
                     mip_map_filter: None,
@@ -178,7 +174,7 @@ impl GUI {
         width: u32,
         height: u32,
         mesh: &egui::paint::Mesh,
-        texture: &Texture2D<u8>,
+        texture: &Texture2D,
     ) -> ThreeDResult<()> {
         debug_assert!(mesh.is_valid());
 
@@ -186,21 +182,21 @@ impl GUI {
         let mut colors = Vec::new();
         let mut uvs = Vec::new();
         for v in mesh.vertices.iter() {
-            positions.push(v.pos.x);
-            positions.push(v.pos.y);
-            uvs.push(v.uv.x);
-            uvs.push(v.uv.y);
-            colors.push(v.color[0] as f32);
-            colors.push(v.color[1] as f32);
-            colors.push(v.color[2] as f32);
-            colors.push(v.color[3] as f32);
+            positions.push(vec2(v.pos.x, v.pos.y));
+            uvs.push(vec2(v.uv.x, 1.0 - v.uv.y));
+            colors.push(vec4(
+                v.color[0] as f32,
+                v.color[1] as f32,
+                v.color[2] as f32,
+                v.color[3] as f32,
+            ));
         }
         let indices: Vec<u32> = mesh.indices.iter().map(|idx| *idx as u32).collect();
 
-        let position_buffer = VertexBuffer::new_with_static(&self.context, &positions)?;
-        let uv_buffer = VertexBuffer::new_with_static(&self.context, &uvs)?;
-        let color_buffer = VertexBuffer::new_with_static(&self.context, &colors)?;
-        let index_buffer = ElementBuffer::new_with(&self.context, &indices)?;
+        let position_buffer = VertexBuffer::new_with_data(&self.context, &positions)?;
+        let uv_buffer = VertexBuffer::new_with_data(&self.context, &uvs)?;
+        let color_buffer = VertexBuffer::new_with_data(&self.context, &colors)?;
+        let index_buffer = ElementBuffer::new_with_data(&self.context, &indices)?;
 
         let render_states = RenderStates {
             blend: Blend::Enabled {
@@ -218,15 +214,16 @@ impl GUI {
 
         self.program.use_texture("u_sampler", texture)?;
         self.program
-            .use_uniform_vec2("u_screen_size", &vec2(width as f32, height as f32))?;
-
-        self.program.use_attribute_vec2("a_pos", &position_buffer)?;
-        self.program.use_attribute_vec4("a_srgba", &color_buffer)?;
-        self.program.use_attribute_vec2("a_tc", &uv_buffer)?;
+            .use_uniform("u_screen_size", vec2(width as f32, height as f32))?;
 
         self.program
-            .draw_elements(render_states, viewport, &index_buffer);
-        Ok(())
+            .use_vertex_attribute("a_pos", &position_buffer)?;
+        self.program
+            .use_vertex_attribute("a_srgba", &color_buffer)?;
+        self.program.use_vertex_attribute("a_tc", &uv_buffer)?;
+
+        self.program
+            .draw_elements(render_states, viewport, &index_buffer)
     }
 }
 
