@@ -3,7 +3,7 @@ use crate::renderer::*;
 use std::rc::Rc;
 
 ///
-/// A physically-based material that renders a [Shadable] object in an approximate correct physical manner based on Physically Based Rendering (PBR).
+/// A physically-based material that renders a [Geometry] in an approximate correct physical manner based on Physically Based Rendering (PBR).
 /// This material is affected by lights.
 ///
 #[derive(Clone)]
@@ -13,39 +13,66 @@ pub struct PhysicalMaterial {
     /// Albedo base color, also called diffuse color. Assumed to be in linear color space.
     pub albedo: Color,
     /// Texture with albedo base colors, also called diffuse color. Assumed to be in sRGB with or without an alpha channel.
-    pub albedo_texture: Option<Rc<Texture2D<u8>>>,
-    /// A value in the range `[0..1]` specifying how metallic the material is.
+    pub albedo_texture: Option<Rc<Texture2D>>,
+    /// A value in the range `[0..1]` specifying how metallic the surface is.
     pub metallic: f32,
-    /// A value in the range `[0..1]` specifying how rough the material surface is.
+    /// A value in the range `[0..1]` specifying how rough the surface is.
     pub roughness: f32,
     /// Texture containing the metallic and roughness parameters which are multiplied with the [Self::metallic] and [Self::roughness] values in the shader.
     /// The metallic values are sampled from the blue channel and the roughness from the green channel.
-    pub metallic_roughness_texture: Option<Rc<Texture2D<u8>>>,
+    pub metallic_roughness_texture: Option<Rc<Texture2D>>,
     /// A scalar multiplier controlling the amount of occlusion applied from the [Self::occlusion_texture]. A value of 0.0 means no occlusion. A value of 1.0 means full occlusion.
     pub occlusion_strength: f32,
     /// An occlusion map. Higher values indicate areas that should receive full indirect lighting and lower values indicate no indirect lighting.
     /// The occlusion values are sampled from the red channel.
-    pub occlusion_texture: Option<Rc<Texture2D<u8>>>,
+    pub occlusion_texture: Option<Rc<Texture2D>>,
     /// A scalar multiplier applied to each normal vector of the [Self::normal_texture].
     pub normal_scale: f32,
     /// A tangent space normal map, also known as bump map.
-    pub normal_texture: Option<Rc<Texture2D<u8>>>,
-    /// Render states used when the color is opaque (has a maximal alpha value).
-    pub opaque_render_states: RenderStates,
-    /// Render states used when the color is transparent (does not have a maximal alpha value).
-    pub transparent_render_states: RenderStates,
-
+    pub normal_texture: Option<Rc<Texture2D>>,
+    /// Render states.
+    pub render_states: RenderStates,
+    /// Whether this material should be treated as a transparent material (An object needs to be rendered differently depending on whether it is transparent or opaque).
+    pub is_transparent: bool,
+    /// Color of light shining from an object.
     pub emissive: Color,
-    pub emissive_texture: Option<Rc<Texture2D<u8>>>,
+    /// Texture with color of light shining from an object.
+    pub emissive_texture: Option<Rc<Texture2D>>,
+    /// The lighting model used when rendering this material
+    pub lighting_model: LightingModel,
 }
 
 impl PhysicalMaterial {
     ///
-    /// Constructs a new physical material from a [CPUMaterial].
-    /// If the input contains an [CPUMaterial::occlusion_metallic_roughness_texture], this texture is used for both
-    /// [PhysicalMaterial::metallic_roughness_texture] and [PhysicalMaterial::occlusion_texture] while any [CPUMaterial::metallic_roughness_texture] or [CPUMaterial::occlusion_texture] are ignored.
+    /// Constructs a new physical material from a [CpuMaterial].
+    /// If the input contains an [CpuMaterial::occlusion_metallic_roughness_texture], this texture is used for both
+    /// [PhysicalMaterial::metallic_roughness_texture] and [PhysicalMaterial::occlusion_texture] while any [CpuMaterial::metallic_roughness_texture] or [CpuMaterial::occlusion_texture] are ignored.
+    /// Tries to infer whether this material is transparent or opaque from the alpha value of the albedo color and the alpha values in the albedo texture.
+    /// Since this is not always correct, it is preferred to use [PhysicalMaterial::new_opaque] or [PhysicalMaterial::new_transparent].
     ///
-    pub fn new(context: &Context, cpu_material: &CPUMaterial) -> ThreeDResult<Self> {
+    pub fn new(context: &Context, cpu_material: &CpuMaterial) -> ThreeDResult<Self> {
+        Self::new_internal(context, cpu_material, super::is_transparent(cpu_material))
+    }
+
+    /// Constructs a new opaque physical material from a [CpuMaterial].
+    /// If the input contains an [CpuMaterial::occlusion_metallic_roughness_texture], this texture is used for both
+    /// [PhysicalMaterial::metallic_roughness_texture] and [PhysicalMaterial::occlusion_texture] while any [CpuMaterial::metallic_roughness_texture] or [CpuMaterial::occlusion_texture] are ignored.
+    pub fn new_opaque(context: &Context, cpu_material: &CpuMaterial) -> ThreeDResult<Self> {
+        Self::new_internal(context, cpu_material, false)
+    }
+
+    /// Constructs a new transparent physical material from a [CpuMaterial].
+    /// If the input contains an [CpuMaterial::occlusion_metallic_roughness_texture], this texture is used for both
+    /// [PhysicalMaterial::metallic_roughness_texture] and [PhysicalMaterial::occlusion_texture] while any [CpuMaterial::metallic_roughness_texture] or [CpuMaterial::occlusion_texture] are ignored.
+    pub fn new_transparent(context: &Context, cpu_material: &CpuMaterial) -> ThreeDResult<Self> {
+        Self::new_internal(context, cpu_material, true)
+    }
+
+    fn new_internal(
+        context: &Context,
+        cpu_material: &CpuMaterial,
+        is_transparent: bool,
+    ) -> ThreeDResult<Self> {
         let albedo_texture = if let Some(ref cpu_texture) = cpu_material.albedo_texture {
             Some(Rc::new(Texture2D::new(&context, cpu_texture)?))
         } else {
@@ -91,21 +118,26 @@ impl PhysicalMaterial {
             normal_scale: cpu_material.normal_scale,
             occlusion_texture,
             occlusion_strength: cpu_material.occlusion_strength,
-            opaque_render_states: RenderStates::default(),
-            transparent_render_states: RenderStates {
-                write_mask: WriteMask::COLOR,
-                blend: Blend::TRANSPARENCY,
-                ..Default::default()
+            render_states: if is_transparent {
+                RenderStates {
+                    write_mask: WriteMask::COLOR,
+                    blend: Blend::TRANSPARENCY,
+                    ..Default::default()
+                }
+            } else {
+                RenderStates::default()
             },
+            is_transparent,
             emissive: cpu_material.emissive,
             emissive_texture,
+            lighting_model: cpu_material.lighting_model,
         })
     }
 }
 
 impl Material for PhysicalMaterial {
-    fn fragment_shader_source(&self, use_vertex_colors: bool, lights: &Lights) -> String {
-        let mut output = lights.fragment_shader_source();
+    fn fragment_shader_source(&self, use_vertex_colors: bool, lights: &[&dyn Light]) -> String {
+        let mut output = lights_fragment_shader_source(lights, self.lighting_model);
         if self.albedo_texture.is_some()
             || self.metallic_roughness_texture.is_some()
             || self.normal_texture.is_some()
@@ -139,51 +171,53 @@ impl Material for PhysicalMaterial {
         &self,
         program: &Program,
         camera: &Camera,
-        lights: &Lights,
+        lights: &[&dyn Light],
     ) -> ThreeDResult<()> {
-        lights.use_uniforms(program, camera)?;
-        program.use_uniform_float("metallic", &self.metallic)?;
-        program.use_uniform_float("roughness", &self.roughness)?;
-        program.use_uniform_vec4("albedo", &self.albedo.to_vec4())?;
-        if program.requires_uniform("emissive") {
-            program.use_uniform_vec3("emissive", &self.emissive.to_vec3())?;
-        }
-        if let Some(ref texture) = self.albedo_texture {
-            program.use_texture("albedoTexture", texture.as_ref())?;
-        }
-        if let Some(ref texture) = self.metallic_roughness_texture {
-            program.use_texture("metallicRoughnessTexture", texture.as_ref())?;
-        }
-        if let Some(ref texture) = self.occlusion_texture {
-            program.use_uniform_float("occlusionStrength", &self.occlusion_strength)?;
-            program.use_texture("occlusionTexture", texture.as_ref())?;
-        }
-        if let Some(ref texture) = self.normal_texture {
-            program.use_uniform_float("normalScale", &self.normal_scale)?;
-            program.use_texture("normalTexture", texture.as_ref())?;
+        if lights.len() > 0 {
+            program.use_uniform_if_required("eyePosition", camera.position())?;
+            for (i, light) in lights.iter().enumerate() {
+                light.use_uniforms(program, i as u32)?;
+            }
+            program.use_uniform("metallic", self.metallic)?;
+            program.use_uniform_if_required("roughness", self.roughness)?;
+            program.use_uniform("albedo", self.albedo)?;
+            if program.requires_uniform("albedoTexture") {
+                if let Some(ref texture) = self.albedo_texture {
+                    program.use_texture("albedoTexture", texture)?;
+                }
+            }
+            if program.requires_uniform("metallicRoughnessTexture") {
+                if let Some(ref texture) = self.metallic_roughness_texture {
+                    program.use_texture("metallicRoughnessTexture", texture)?;
+                }
+            }
+            if program.requires_uniform("occlusionTexture") {
+                if let Some(ref texture) = self.occlusion_texture {
+                    program.use_uniform("occlusionStrength", self.occlusion_strength)?;
+                    program.use_texture("occlusionTexture", texture)?;
+                }
+            }
+            if program.requires_uniform("normalTexture") {
+                if let Some(ref texture) = self.normal_texture {
+                    program.use_uniform("normalScale", self.normal_scale)?;
+                    program.use_texture("normalTexture", texture)?;
+                }
+            }
         }
+        program.use_uniform("emissive", self.emissive)?;
         if program.requires_uniform("emissiveTexture") {
             if let Some(ref texture) = self.emissive_texture {
-                program.use_texture("emissiveTexture", texture.as_ref())?;
+                program.use_texture("emissiveTexture", texture)?;
             }
         }
         Ok(())
     }
 
     fn render_states(&self) -> RenderStates {
-        if self.is_transparent() {
-            self.transparent_render_states
-        } else {
-            self.opaque_render_states
-        }
+        self.render_states
     }
     fn is_transparent(&self) -> bool {
-        self.albedo.a != 255
-            || self
-                .albedo_texture
-                .as_ref()
-                .map(|t| t.is_transparent())
-                .unwrap_or(false)
+        self.is_transparent
     }
 }
 
@@ -200,14 +234,11 @@ impl Default for PhysicalMaterial {
             normal_scale: 1.0,
             occlusion_texture: None,
             occlusion_strength: 1.0,
-            opaque_render_states: RenderStates::default(),
-            transparent_render_states: RenderStates {
-                write_mask: WriteMask::COLOR,
-                blend: Blend::TRANSPARENCY,
-                ..Default::default()
-            },
+            render_states: RenderStates::default(),
+            is_transparent: false,
             emissive: Color::BLACK,
             emissive_texture: None,
+            lighting_model: LightingModel::Blinn,
         }
     }
 }
