@@ -37,57 +37,101 @@ mod canvas;
 pub use canvas::*;
 
 /// Type of mouse button.
-#[allow(missing_docs)]
 #[derive(Clone, Copy, Debug, Eq, Ord, PartialEq, PartialOrd, Hash)]
 pub enum MouseButton {
+    /// Left mouse button or one finger on touch.
     Left,
+    /// Left mouse button or two fingers on touch.
     Right,
+    /// Middle mouse button.
     Middle,
 }
 
 /// An input event (from mouse, keyboard or similar).
 #[derive(Clone, Debug)]
 pub enum Event {
+    /// Fired when a button is pressed or the screen is touched.
     MousePress {
+        /// Type of button
         button: MouseButton,
+        /// The screen position in logical pixels, to get it in physical pixels, multiply it with [FrameInput::device_pixel_ratio].
+        /// The first value defines the position on the horizontal axis with zero being at the left border of the window
+        /// and the second on the vertical axis with zero being at the top edge of the window.
         position: (f64, f64),
+        /// The state of modifiers.
         modifiers: Modifiers,
+        /// Whether or not this event already have been handled.
         handled: bool,
     },
+    /// Fired when a button is released or the screen is stopped being touched.
     MouseRelease {
+        /// Type of button
         button: MouseButton,
+        /// The screen position in logical pixels, to get it in physical pixels, multiply it with [FrameInput::device_pixel_ratio].
+        /// The first value defines the position on the horizontal axis with zero being at the left border of the window
+        /// and the second on the vertical axis with zero being at the top edge of the window.
         position: (f64, f64),
+        /// The state of modifiers.
         modifiers: Modifiers,
+        /// Whether or not this event already have been handled.
         handled: bool,
     },
+    /// Fired continuously when the mouse or a finger on the screen is moved.
     MouseMotion {
+        /// Type of button if a button is pressed.
         button: Option<MouseButton>,
+        /// The relative movement of the mouse/finger since last [Event::MouseMotion] event.
         delta: (f64, f64),
+        /// The screen position in logical pixels, to get it in physical pixels, multiply it with [FrameInput::device_pixel_ratio].
+        /// The first value defines the position on the horizontal axis with zero being at the left border of the window
+        /// and the second on the vertical axis with zero being at the top edge of the window.
         position: (f64, f64),
+        /// The state of modifiers.
         modifiers: Modifiers,
+        /// Whether or not this event already have been handled.
         handled: bool,
     },
+    /// Fired continuously when the mouse wheel or equivalent is applied.
     MouseWheel {
+        /// The relative scrolling since the last [Event::MouseWheel] event.
         delta: (f64, f64),
+        /// The screen position in logical pixels, to get it in physical pixels, multiply it with [FrameInput::device_pixel_ratio].
+        /// The first value defines the position on the horizontal axis with zero being at the left border of the window
+        /// and the second on the vertical axis with zero being at the top edge of the window.
         position: (f64, f64),
+        /// The state of modifiers.
         modifiers: Modifiers,
+        /// Whether or not this event already have been handled.
         handled: bool,
     },
+    /// Fired when the mouse enters the window.
     MouseEnter,
+    /// Fired when the mouse leaves the window.
     MouseLeave,
+    /// Fired when a key is pressed.
     KeyPress {
+        /// The type of key.
         kind: Key,
+        /// The state of modifiers.
         modifiers: Modifiers,
+        /// Whether or not this event already have been handled.
         handled: bool,
     },
+    /// Fired when a key is released.
     KeyRelease {
+        /// The type of key.
         kind: Key,
+        /// The state of modifiers.
         modifiers: Modifiers,
+        /// Whether or not this event already have been handled.
         handled: bool,
     },
+    /// Fired when the modifiers change.
     ModifiersChange {
+        /// The state of modifiers after the change.
         modifiers: Modifiers,
     },
+    /// Fires when some text has been written.
     Text(String),
 }
 
@@ -213,8 +257,9 @@ pub struct FrameInput {
 #[derive(Clone, Debug)]
 pub struct FrameOutput {
     ///
-    /// Closes the window and stops the renderloop if this is true.
-    /// Only relevant on desktop, ignored on web.
+    /// If this is true:
+    /// - On desktop, the window is closed and the renderloop is stopped.
+    /// - On web, the render loop is stopped, the event handlers are removed and the `Window` dropped. Note that the canvas is not removed.
     ///
     pub exit: bool,
 
