@@ -35,6 +35,9 @@ pub enum CanvasError {
     EventListenerFail(String, String),
 }
 
+///
+/// Default window (canvas) and event handler for easy setup.
+///
 pub struct Window {
     canvas: Option<web_sys::HtmlCanvasElement>,
     window: Rc<web_sys::Window>,
@@ -48,6 +51,9 @@ pub struct Window {
 }
 
 impl Window {
+    ///
+    /// Constructs a new window with the given settings.
+    ///
     pub fn new(settings: WindowSettings) -> ThreeDResult<Window> {
         let websys_window = web_sys::window().ok_or(CanvasError::WindowCreation)?;
         let document = websys_window
@@ -94,16 +100,25 @@ impl Window {
         Ok(())
     }
 
+    ///
+    /// Return the current logical size of the window.
+    ///
     pub fn size(&self) -> ThreeDResult<(u32, u32)> {
         let canvas = self.canvas.as_ref().ok_or(CanvasError::CanvasMissing)?;
         Ok((canvas.width(), canvas.height()))
     }
 
+    ///
+    /// Returns the current viewport of the window in physical pixels (the size of the [screen](crate::Screen)).
+    ///
     pub fn viewport(&self) -> ThreeDResult<Viewport> {
         let (w, h) = self.size()?;
         Ok(Viewport::new_at_origo(w, h))
     }
 
+    ///
+    /// Returns the graphics context for this window.
+    ///
     pub fn gl(&self) -> ThreeDResult<Context> {
         let context_options = ContextOptions {
             antialias: self.settings.multisamples > 0,
@@ -130,11 +145,14 @@ impl Window {
             .get_extension("OES_texture_float_linear")
             .map_err(|e| CanvasError::OESTextureFloatNotSupported(format!(": {:?}", e)))?;
 
-        Ok(crate::core::Context::from_gl_context(
-            crate::context::GLContext::new(context),
+        crate::core::Context::from_gl_context(Rc::new(
+            crate::context::Context::from_webgl2_context(context),
         ))
     }
 
+    ///
+    /// Start the main render loop which calls the `callback` closure each frame.
+    ///
     pub fn render_loop<F: 'static + FnMut(FrameInput) -> FrameOutput>(
         mut self,
         mut callback: F,
@@ -186,7 +204,9 @@ impl Window {
             first_frame = false;
             let frame_output = callback(frame_input);
 
-            if !frame_output.wait_next_event {
+            if frame_output.exit {
+                input_clone.borrow_mut().render_loop_closure = None;
+            } else if !frame_output.wait_next_event {
                 input_clone.borrow_mut().request_animation_frame();
             }
         })
@@ -270,7 +290,10 @@ impl Window {
             event.stop_propagation();
         }) as Box<dyn FnMut(_)>);
         self.canvas()?
-            .set_oncontextmenu(Some(closure.as_ref().unchecked_ref()));
+            .add_event_listener_with_callback("contextmenu", closure.as_ref().unchecked_ref())
+            .map_err(|e| {
+                CanvasError::EventListenerFail("contextmenu".to_string(), format!("{:?}", e))
+            })?;
         self.closures_with_event.push(closure);
         Ok(())
     }
@@ -597,8 +620,6 @@ impl Window {
                 if update_modifiers(&mut input.modifiers, &event) {
                     let modifiers = input.modifiers;
                     input.events.push(Event::ModifiersChange { modifiers });
-                    event.stop_propagation();
-                    event.prevent_default();
                 }
                 let key = event.key();
                 let modifiers = input.modifiers;
@@ -608,14 +629,9 @@ impl Window {
                         modifiers,
                         handled: false,
                     });
-                    event.stop_propagation();
-                    event.prevent_default();
                 }
                 if !modifiers.ctrl && !modifiers.command && !should_ignore_key(&key) {
                     input.events.push(Event::Text(key));
-                    event.stop_propagation();
-                    event.prevent_default();
-
                     input.request_animation_frame();
                 }
             }
@@ -639,8 +655,6 @@ impl Window {
                 if update_modifiers(&mut input.modifiers, &event) {
                     let modifiers = input.modifiers;
                     input.events.push(Event::ModifiersChange { modifiers });
-                    event.stop_propagation();
-                    event.prevent_default();
                 }
                 if let Some(kind) = translate_key(&event.key()) {
                     let modifiers = input.modifiers;
@@ -649,9 +663,6 @@ impl Window {
                         modifiers,
                         handled: false,
                     });
-                    event.stop_propagation();
-                    event.prevent_default();
-
                     input.request_animation_frame();
                 }
             }
@@ -720,6 +731,102 @@ impl Input {
     }
 }
 
+impl Drop for Window {
+    fn drop(&mut self) {
+        if let Some(closure) = self.closures_with_event.pop() {
+            self.canvas()
+                .unwrap()
+                .remove_event_listener_with_callback(
+                    "contextmenu",
+                    closure.as_ref().unchecked_ref(),
+                )
+                .unwrap();
+        }
+
+        if let Some(closure) = self.closures.pop() {
+            self.canvas()
+                .unwrap()
+                .remove_event_listener_with_callback("resize", closure.as_ref().unchecked_ref())
+                .unwrap();
+        }
+
+        if let Some(closure) = self.closures_with_mouseevent.pop() {
+            self.canvas()
+                .unwrap()
+                .remove_event_listener_with_callback("mousemove", closure.as_ref().unchecked_ref())
+                .unwrap();
+        }
+        if let Some(closure) = self.closures_with_mouseevent.pop() {
+            self.canvas()
+                .unwrap()
+                .remove_event_listener_with_callback("mouseup", closure.as_ref().unchecked_ref())
+                .unwrap();
+        }
+        if let Some(closure) = self.closures_with_mouseevent.pop() {
+            self.canvas()
+                .unwrap()
+                .remove_event_listener_with_callback("mousedown", closure.as_ref().unchecked_ref())
+                .unwrap();
+        }
+        if let Some(closure) = self.closures_with_mouseevent.pop() {
+            self.canvas()
+                .unwrap()
+                .remove_event_listener_with_callback("mouseleave", closure.as_ref().unchecked_ref())
+                .unwrap();
+        }
+        if let Some(closure) = self.closures_with_mouseevent.pop() {
+            self.canvas()
+                .unwrap()
+                .remove_event_listener_with_callback("mouseenter", closure.as_ref().unchecked_ref())
+                .unwrap();
+        }
+
+        if let Some(closure) = self.closures_with_wheelevent.pop() {
+            self.canvas()
+                .unwrap()
+                .remove_event_listener_with_callback("wheel", closure.as_ref().unchecked_ref())
+                .unwrap();
+        }
+
+        if let Some(closure) = self.closures_with_touchevent.pop() {
+            self.canvas()
+                .unwrap()
+                .remove_event_listener_with_callback("touchmove", closure.as_ref().unchecked_ref())
+                .unwrap();
+        }
+        if let Some(closure) = self.closures_with_touchevent.pop() {
+            self.canvas()
+                .unwrap()
+                .remove_event_listener_with_callback("touchend", closure.as_ref().unchecked_ref())
+                .unwrap();
+        }
+        if let Some(closure) = self.closures_with_touchevent.pop() {
+            self.canvas()
+                .as_mut()
+                .unwrap()
+                .remove_event_listener_with_callback("touchstart", closure.as_ref().unchecked_ref())
+                .unwrap();
+        }
+
+        if let Some(closure) = self.closures_with_keyboardevent.pop() {
+            web_sys::window()
+                .unwrap()
+                .document()
+                .unwrap()
+                .remove_event_listener_with_callback("keyup", closure.as_ref().unchecked_ref())
+                .unwrap();
+        }
+        if let Some(closure) = self.closures_with_keyboardevent.pop() {
+            web_sys::window()
+                .unwrap()
+                .document()
+                .unwrap()
+                .remove_event_listener_with_callback("keydown", closure.as_ref().unchecked_ref())
+                .unwrap();
+        }
+    }
+}
+
 fn update_modifiers(modifiers: &mut Modifiers, event: &web_sys::KeyboardEvent) -> bool {
     let old = modifiers.clone();
     *modifiers = Modifiers {
